import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * A movable block, move blocks onto all the spot to complete the level.
 */
public class Block extends Actor
{
    
    private boolean isMoving;
    private boolean isOnSpot = false;
    
    private int moveTimer = 4000000;
    private int x, y;
    
    private GreenfootImage img_notOnSpot = new GreenfootImage("block.png");
    private GreenfootImage img_isOnSpot = new GreenfootImage("block_spot.png");
    
    public Block(){
        
    }
    
    public void act() 
    {
        setLocation(x,y);
        killEnemyOnTheWay();
    }    
    
    public void addedToWorld(World world){
        x = getX();
        y = getY();
        
        if (getOneIntersectingObject(Spot.class) != null){
             isOnSpot = true;
                 changeColor();
                 ((Level)getWorld()).getCounter().setBlockOnSpotCount(((Level)getWorld()).getCounter().getBlockOnSpotCount()+1);
        }
    }
    
    public void move(int dir){
        if (checkMovable(dir)[0]){
             if(checkMovable(dir)[1] ){
                isOnSpot = true;
                 changeColor();
                 ((Level)getWorld()).getCounter().setBlockOnSpotCount(((Level)getWorld()).getCounter().getBlockOnSpotCount()+1);
                 Sound.moveBlockIntoSpot();
           } else {
               if (isOnSpot){
                   //delete score once out of the spot
                   ((Level)getWorld()).getCounter().setBlockOnSpotCount(((Level)getWorld()).getCounter().getBlockOnSpotCount()-1);
                   Sound.moveBlockOutOfSpot();
                }
               isOnSpot = false;
               changeColor();
                
            }
            if(dir == 0){
            while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        y++;
                    }
                    moveTimer--;
                }
            moveTimer = 4000000;
        } else if (dir == 1){
            while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        x--;
                    }
                    moveTimer--;
                }
            moveTimer = 4000000;
        } else if (dir == 2){
            while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        x++;
                    }
                    moveTimer--;
                }
            moveTimer = 4000000;
        } else if (dir == 3){
            while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        y--;
                    }
                    moveTimer--;
                }
            moveTimer = 4000000;
        }
        
        Sound.moveBlock();
      }
      
    }
    
    public boolean[] checkMovable(int dir){
        if(((Level)getWorld()).levelComplete()){
            return new boolean[]{false,false};
        }
        
        boolean movable = false;
        boolean colorChanger = false;
        List list = getObjectsNextTo(dir);
        for(int i = 0; i < list.size(); i++){
            if(list.get(i).getClass() == Space.class){
                movable = true;
            } 
            else if (list.get(i).getClass() == Spot.class){
                colorChanger = true;
            }
            else if (list.get(i).getClass() != Space.class && list.get(i).getClass() != Spot.class
            && list.get(i).getClass() != Slime.class && list.get(i).getClass() != Goblin.class
            && list.get(i).getClass() != LevelComplete.class && list.get(i).getClass() != Explosion.class){
                return new boolean[]{false,false};
            }
            
        }
        
        return new boolean[]{movable,colorChanger};
    }
    
    public List getObjectsNextTo(int dir){
        if(dir == 0){
            return this.getObjectsAtOffset(0, 4, Actor.class);
        } else if (dir == 1){
            return this.getObjectsAtOffset(-4, 0, Actor.class);
        } else if (dir == 2){
            return this.getObjectsAtOffset(4, 0, Actor.class);
        } else if (dir == 3){
            return this.getObjectsAtOffset(0, -4, Actor.class);
        } else {
            return null;
        }
    }
    
    public boolean checkOnSpot(){
        return isOnSpot;
    }
    
    public void changeColor(){
        if(isOnSpot){
            setImage(img_isOnSpot);
            
        } else {
            setImage(img_notOnSpot);
            
        }
    }
    
    public void killEnemyOnTheWay(){
        Enemy enemy = (Enemy)getOneObjectAtOffset(0, 0, Enemy.class);
        if(enemy != null){
            enemy.die();
        }
    }
    
}
