import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.event.KeyListener;
import java.util.List;

/**
 * Player class is the super class for every playble characters' class
 * The input setting is found here.
 * 
 * Move in 32x32 spot
 * change sprites according to the player type (Boy or Girl)
 * 
 */
public abstract class Player extends Actor implements Mortal
{
    
    /**Sprite Images for each player*/
    protected GreenfootImage[] spr = new GreenfootImage[12];
    //0,1,2 down
    //3,4,5 left
    //6,7,8 right
    //9,10,11 up
    
    /**Movement checker*/
    protected boolean isMoving;
    protected boolean canMove;
    protected int moveTimer = 4000000;
    
    /**Status checker*/
    protected boolean dead = false;
    protected boolean frozen = false;
    protected GreenfootImage death = new GreenfootImage("graveyard.png");
    
    /**2D coordinates*/
    protected int x, y;
    
    /**Facing Direction Sprite Trigger*/
    protected int facing_dir = 0;
    protected int spr_index = 1;
    protected int spr_i = 0;
    
    /**Delay Trigger*/
    protected int move_delay = 5;
    protected int shoot_delay = 10;
    
    /**
     * Initialize Player
     */
    public Player(){
        isMoving = false;
        canMove = true;
    }
    
    /**
     * Act, by move with keyboard input.
     */
    public void act() 
    {
        if(canMove && !dead && move_delay < 0){
            input();
            animate();
        }
        move_delay--;
        killEnemyWhenTouched();
        setLocation(x,y);
    }    
    
    /**
     * Move with keyboard, this method also use checkMovable(int) to check for obstacles on the specific direction
     */
    private void input(){
        if ((Greenfoot.isKeyDown("down") && this instanceof Boy)||
        (Greenfoot.isKeyDown("s") && this instanceof Girl)){
            if(facing_dir == 0){
            if(checkMovable(0) && canMove){
                isMoving = true;
                canMove = false;
                while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        y++;
                    }
                    moveTimer--;
                }
                moveTimer = 4000000;
                isMoving = false;
                canMove = true;
                spr_index = 1;
                walkTriggerAnim();
            } else {
                spr_index = 1;
                spr_i = 0;
            }
          } else {
              facing_dir = 0;
            }
            
        } 
        else if ((Greenfoot.isKeyDown("left") && canMove && this instanceof Boy)||
        (Greenfoot.isKeyDown("a") && canMove && this instanceof Girl)){
            if (facing_dir == 1){
            if(checkMovable(1)){
                isMoving = true;
                canMove = false;
                while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        x--;
                    }
                    moveTimer--;
                }
                moveTimer = 4000000;
                isMoving = false;
                canMove = true;
                spr_index = 4;
                walkTriggerAnim();
            } else {
                spr_index = 4;
                spr_i = 0;
            }
          } else {
              facing_dir = 1;
            }
            
        }
        else if ((Greenfoot.isKeyDown("right") && canMove && this instanceof Boy)||
        (Greenfoot.isKeyDown("d") && canMove && this instanceof Girl)){
            if (facing_dir == 2){
            if(checkMovable(2)){
               isMoving = true;
               canMove = false;
               while(moveTimer > 0){
                   if(moveTimer % 1000000 == 0){
                       x++;
                    }
                    moveTimer--;
                }
                moveTimer = 4000000;
                isMoving = false;
                canMove = true;
                spr_index = 7;
                walkTriggerAnim(); 
            } else {
                spr_index = 7;
                spr_i = 0;
            }
           } else {
               facing_dir = 2;
            }
            
        }
        else if ((Greenfoot.isKeyDown("up") && canMove && this instanceof Boy)||
        (Greenfoot.isKeyDown("w") && canMove && this instanceof Girl)){
            if (facing_dir == 3){
            if(checkMovable(3)){
                isMoving = true;
                canMove = false;
                while(moveTimer > 0){
                    if(moveTimer % 1000000 == 0){
                        y--;
                    }
                    moveTimer--;
                }
                moveTimer = 4000000;
                isMoving = false;
                canMove = true;
                spr_index = 10;
                walkTriggerAnim();
            } else {
                spr_index = 10;
                spr_i = 0;
            }
          }
          else {
              facing_dir = 3;
            }
        }
    }
    
    /**
     * Animation
     */
    private void animate(){
        setImage(spr[spr_index+spr_i]);
    }
    /**
     * Trigger Walking Animation
     */
    private void walkTriggerAnim(){
        if(spr_i == 0){
            spr_i = 1;
        }
        else if(spr_i == 1){
            spr_i = -1;
        } else if(spr_i == -1){
            spr_i = 1;
        }
        move_delay = 5;
        
    }
    
    /**
     * Add player to the world
     */
    public void addedToWorld(World world){
        x = getX();
        y = getY();
    }
   
    
    
    /**
     * Check for objects in the way.
     * Make sure no objects is in the way in the specific direction so the player can move (true value)
     * dir = 0 ; down
     * dir = 1 ; left
     * dir = 2 ; right
     * dir = 3 ; up
     */
    public boolean checkMovable(int dir){
        boolean movable = false;
        List list = getObjectsNextTo(dir);
        for(int i = 0; i < list.size(); i++){
            if(list.get(i).getClass() == Space.class){
                movable = true;
            } else if (list.get(i).getClass() == Block.class ){
                if(((Block)list.get(i)).checkMovable(dir)[0]){
                    pushBlock((Block)list.get(i),dir);
                }
                else {
                    return false;
                }
            }
            else if (list.get(i).getClass() != Space.class && list.get(i).getClass() != Spot.class
            && list.get(i).getClass() != Block.class && list.get(i).getClass() != Slime.class && 
            list.get(i).getClass() != Explosion.class && list.get(i).getClass() != Goblin.class &&
            list.get(i).getClass() != LevelComplete.class){
                return false;
            }
            
        }
        
        return movable;
    }
    
    /**
     * Push block interaction (used in checkMovable(int))
     */
    private void pushBlock(Block block, int dir){
        block.move(dir);
    }
    
    /**
     * Get Lits of objects nearby for 1 space unit (used in checkMovable(int))
     * dir = 0 ; down
     * dir = 1 ; left
     * dir = 2 ; right
     * dir = 3 ; up
     */
    public List getObjectsNextTo(int dir){
        if(dir == 0){
            return this.getObjectsAtOffset(0, 4, Actor.class);
        } else if (dir == 1){
            return this.getObjectsAtOffset(-4, 0, Actor.class);
        } else if (dir == 2){
            return this.getObjectsAtOffset(4, 0, Actor.class);
        } else if (dir == 3){
            return this.getObjectsAtOffset(0, -4, Actor.class);
        } else {
            return null;
        }
    }
    
    /**
     * Once the enemy touches the player, kill that enemy
     */
    private void killEnemyWhenTouched(){
        Enemy enemy = (Enemy)getOneObjectAtOffset(0, 0, Enemy.class);
        if(enemy != null){
            enemy.die();
        }
    }
    
    /**
     * Death (implementation of Mortal interface class)
     */
    public void die(){
        if(!dead){
           dead = true;
           setImage(death);
           if(this instanceof Boy){
               Sound.boyDie();
            } else if (this instanceof Girl){
                Sound.girlDie();
            } 
        }
        
    }
    
    /**
     * Un-Death (only for Player.class)
     */
    public void revive(){
        if (dead){
            dead = false;
            setImage(spr[1]);
            Sound.revive();
        }
        
    }
    
    /**
     * Freeze
     */
    public void freeze(){
        frozen = true;
        canMove = false;
    }
    
    /**
     * Unfreeze
     */
    public void unfreeze(){
        frozen = false;
        canMove = true;
    }
}
